/************************************************************************************************
*                                         SENTINEL PROGRAM
*************************************************************************************************
* NAME:  
*
* PURPOSE:
*
* MAJOR STEPS: 
*
* KEY DEPENDENCIES/CONSTRAINTS/CAVEATS:
*
*------------------------------------------------------------------------------------------------
* CONTACT INFO: 
*  Sentinel Coordinating Center
*  info@sentinelsystem.org
*
*------------------------------------------------------------------------------------------------
* HISTORY:
*  Create date (mm/dd/yy):
*  Last modified date (mm/dd/yy): 
*  Version: 
*
************************************************************************************************/

/*-----------------------------------------------------------------------------------------------
*                                        DEFINITIONS
*------------------------------------------------------------------------------------------------
* SCC   ... Sentinel Common-Components File 
*           Specific to each Data Partner/Data Site, the Common-Components defines key
*               - macro variables (e.g. Data Partner ID, Min and Max Dates, SCDM table names)
*               - libname assignments (e.g. libref INDATA is assigned to production SCDM tables)
*             NOTE: This parameter needs to be set by programmer at the Data Partner/Data Site
*
* MSReqID... Sentinel Project Request-ID
*             NOTE: This parameter needs to be set by the programmer preparing the Request-ID
*                   in coordination with the Sentinel Operations Center (SOC)
*
* The following paths for the current request are automatically derived 
*    DPLOCAL  ... path for data created that remains local (i.e. behind the DP firewall)
*    MSOC     ... path for data created to share with Sentinel Operations Center
*    INFOLDER ... path for program inputs (e.g. lookup data, include programs, include macros)
*    SASPROGRAMS ... path for the main SAS program for the request
*----------------------------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------------------------*/
* Define all macro parameters as global - DO NOT EDIT;
	%global dpname etl dp_mindate dp_maxdate dpid siteid enrtable demtable distable diatable 
			proctable enctable _root_dplocal _root_msoc _root_inputfiles _root_sasprograms  
			deathtable codtable labtable vittable miltable ipharmtable itranstable dplocal msoc 
			infolder sasprograms _qaresults ccbypass;
/*---------------------------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------------------------*/
/* Section 1: Data Partner/Data Site programmer defines parameters in this section.			   */
/*---------------------------------------------------------------------------------------------*/

/* 1a. Organizations without a Common Components file should leave blank and skip to 1b		   */
	/* Define Common Components Include File                                                   */
	/* Example: //sentinel/common-components/ms_common_components.sas                          */

	%let SCC = ;

	/*Run Common Components - Do not edit*/
	%macro runcc();
	%if %str("&SCC.") ne %str("") %then %do;
		%inc "&SCC" /source2 ;
	%end;
	%mend;
	%runcc();

/* 1b. Program-specific parameters                                                             */

	/* Edit this parameter to identify the list of patients to exclude (if applicable).
	   Dataset should contain one variable, patid, and list all patids that should be
	   excluded. Must include libname (e.g., indata.PtsToExclude).
	 */
	%let PTSTOEXCLUDE=;

	/* Edit this parameter to make it point to the folder containing dplocal datasets
	   from the prior runs (Prior DPLocal) 
	*/
	%let DPLPrior=;   * Enter the path to the prior run data; 


/* 1c. Organizations without a Common Components file, define parameters in this section 	   */

	/* DPID and SITEID are descriptive identifiers for your organization. 					   */
	/* Specify a 2 character abbreviation for DPID and a 1-4 character abbreviation for SITEID */
	/* Both DPID and SITEID should be uppercase												   */

	%let _DPID= ;   
	%let _SITEID= ;     

	/* Specify the location of the the current request package 
	   (i.e. the folder containing the request dplocal and msoc folders)                       */
	/* For example, %let packageroot = A:/sentinel/cder_ahr_wp005_nsdp_b03                     */												 

	%let _packageroot = ;

	/* Specify the file path for the location of your Sentinel Common Data Model datasets */

	%let prod_scdm=;

	/* The following metadata describe characteristics of your Sentinel Common Data Model      */
	/* If known, modify the following parameters:											   */
	/* DP_MinDate = Please enter the overall SCDM minimum date in the format 'DDMMMYYYY'd.     */
	/*	            Default value is '01Jan2000'd                                              */
	/* DP_MaxDate = Please enter the overall SCDM maximum date in the format 'DDMMMYYYY'd.     */
	/*	            Default value is '31Dec2099'd                                              */
	/* ETL = Please enter the ETL version # for your current production SCDM. Default is 1	   */
	%let _DP_MinDate = '01Jan2000'd ;
	%let _DP_MaxDate = '31Dec2099'd ;
	%let _ETL = 1;

	/* Specify the name of each of the Sentinel Common Data Model tables.        	           */
	/* Do NOT include .sas7bdat 	                                                           */
	/* For example, %let ENRTABLE=enrollment 												   */

	%let _ENRTABLE=;    *specify the enrollment table name    *;        
	%let _DEMTABLE=;   	*specify the demographic table name   *;        
	%let _DISTABLE=;    *specify the rx dispensing table name *;        
	%let _DIATABLE=;    *specify the diagnosis table name     *;        
	%let _PROCTABLE=;   *specify the procedure table name     *;        
	%let _ENCTABLE=;    *specify the encounter table name     *;  

	*********************************************************************************************
	The following tables are optional.  Leave blank if they are not available at your site.  
	Ensure the macro variables below are not commented out even if the table does NOT exist.
	********************************************************************************************;

	%let _DEATHTABLE=;		*specify the death table name          		  *;        
	%let _CODTABLE=;    	*specify the cause of death table name 		  *;              
	%let _LABTABLE=;     	*specify the lab result table name     		  *;        
	%let _VITTABLE=;     	*specify the vital signs table name    		  *;        
	%let _IPHARMTABLE=;		*specify the inpatient pharmacy table name 	  *;
	%let _ITRANSTABLE=;  	*specify the inpatient transfusion table name *;
	%let _MILTABLE=;	    *specify the mother-infant linkage table name *;

	/* When executing the SCDM QA package, specify the directory containing the ETL (SCDM) in  */
	/* process used by the SOC QA programs. Note: Can be blank, do NOT comment out             */
	%let _qa_staging=;  

	/* If available, specify the directory/folder containing the QA result datasets specific   */
	/* to the most recent production ETL                                                       */
	%let _prod_qaresults= ;

/*-------------------------------------------------------------------------------------------------*/
/* Section 2: Analyst preparing Request-ID defines the each parameter in coordination with the SOC */
/* Specific requests are made up a project-ID, workplan-type, workplan-ID, unique-DPID, version-ID */   
/*-------------------------------------------------------------------------------------------------*/
*
*        If this is your request   . . . . .  then set parameter values as follows 
*   ---------------------------------     -------------------------------------------------------
*   [Project-ID: CDER]                        %let MSProjID = cder ;  
*   [Workplan-Type: ad hoc request]           %let MSWPType = ahr ; 
*   [Workplan-ID: 5]                          %let MSWPID =   wp005 ; 
*   [Unique-DPID: non-specific DP]            %let MSDPID =   nsdp ;     
*   [Version-ID: beta 3]                      %let MSVerID =  b03 ;         
*   [use underscores as delimiter]            %let dlm = _ ;                
*
*   The example values above would produce a Request-ID equal to cder_ahr_wp005_nsdp_b03
*
/*---------------------------------------------------------------------------------------------*/

* Specify project-ID, workplan-type, workplan-ID, workplan-type, unique-dpid, version-ID *;
%let MSProjID = edit-project-id ; 
%let MSWPType = edit-workplantype-id ;      
%let MSWPID   = edit-workplan-id ;        
%let MSDPID   = edit-unique DP-id ; /* Note: set value to nsdp for non-specific DP type requests*/
%let MSVerID  = edit-version-id ;        
%let dlm = _ ; 


*************************************************************************************************
*******                                 END OF USER INPUT                                  ******
*******                            DO NOT EDIT BELOW THIS LINE                             ******
*******     (Consult with SOC or Sentinel team leader if you feel edits are required)	   ******
*************************************************************************************************;

*-------------------------------------------------------------------------------------------------
* Section 3a: Setup Environment
*-------------------------------------------------------------------------------------------------;
/*---------------------------------------------------------------------*/
/* NOTE: This is standard SOC environment setup code -- Do Not Edit    */
/*---------------------------------------------------------------------*/

/* System options */
options nosymbolgen;
options nomlogic 
options ls=100 nocenter ;
options obs=MAX ;
options msglevel=i ;
options mprint mprintnest ;
options errorcheck=strict errors=0 ;
options merror serror ;
options dkricond=error dkrocond=error mergenoby=warn;
options dsoptions=nonote2err noquotelenmax ;
options reuse=no ;
options fullstimer ;
options missing = .;

*-------------------------------------------------------------------------------------------------
* Section 3b: Define macro variables and set paths
*-------------------------------------------------------------------------------------------------;

* Request-ID (must be lower case) *;
%let MSReqID = %lowcase(&MSProjID.&dlm.&MSWPType.&dlm.&MSWPID.&dlm.&MSDPID.&dlm.&MSVerID);

* Delete macro variable used as delimiter as we no longer need it *;
%symdel dlm ; 


%macro assignrefs();

*-------------------------------------------------------------------------------------------------
* If common components not specified, define all file paths;
*-------------------------------------------------------------------------------------------------;

%if %str("&SCC.") = %str("") %then %do;

	*Include standard Common Components Macros;
	%macro ms_clean_path(path) ;
		%local ln lc d_exist ;
		%let ln = %length(&path) ;
		%if &ln eq 0 %then %do ;
			 %put The parameter path must be non-missing. ;
			 %abort cancel ;
		%end ;
		%else %if &ln > 0 %then %do ;
		   %let path = %sysfunc(translate(&path, %str(/), %str(\))) ;
		   %let lc = %substr(&path, &ln) ;
		   %if "&lc" ne "/" %then %do ;
		   	  %let path = &path./ ;
		   %end ;	   
		   %let d_exist = %ms_dirExist(&path) ;  
	     %if &d_exist eq 0 %then %do ;
	        %put Path &path does not exist;
	        %abort cancel;
	     %end ;	   
		%end ;
		&path 	 /* returns value to calling enivornment, like a function */
	%mend ms_clean_path ;
	%macro ms_lib(ref, path, options=) ;
	  %if %length(&ref) eq 0 %then %do ;
	     %put libref is blank ;
		 %abort cancel ;
	  %end ;
	  %local d_exist ;
	  %if %length(&path) eq 0 %then %do ;
	     %put MS-NOTE: for libref &ref the path is blank, libname assignment skipped ;
	  %end ;
	  %else %do ;
	     %let d_exist = %ms_dirExist(&path) ;  
	     %if &d_exist eq 1 %then %do ;
	        libname &ref "&path" &options ;
	     %end ;
	     %else %do ;
	        %put Path &path does not exist;
	        %abort cancel;
	     %end ;
	  %end ;
	%mend ms_lib;
	%macro ms_dirExist(dir) ; 
	   %if %length(&dir) eq 0 %then %do ;
	   	  %put The parameter dir must be non-missing.  ;
	   	  %abort cancel ;
	   %end ;
	   %local rc fileref return ; 
	   %let rc = %sysfunc(filename(fileref,&dir)) ; 
	   %let return = %sysfunc(fexist(&fileref)) ;  
	   &return  /* returns value to calling enivornment, like a function */
	   %let rc = %sysfunc(filename(fileref)) ;  
	%mend ms_dirExist;

	/* Define request specific subdirectories */
	%let DPLOCAL = &_packageroot./dplocal/ ; 
	%let MSOC = &_packageroot./msoc/ ;            
	%let INFOLDER = &_packageroot./inputfiles/ ;  
	%let SASPROGRAMS = &_packageroot./sasprograms/ ;                 

	/* Clean up paths (e.g. convert slashes and add slashes when necessary)
	   and validate that paths exist */
	%let DPLOCAL = %ms_clean_path(&DPLOCAL) ;
	%let MSOC = %ms_clean_path(&MSOC) ;
	%let INFOLDER = %ms_clean_path(&INFOLDER) ;
	%let SASPROGRAMS = %ms_clean_path(&SASPROGRAMS) ;
	%if %str("&_qa_staging.") ne %str("") %then %do;
	%let _qa_staging = %ms_clean_path(&_qa_staging) ;
	%end;
	%if %str("&_prod_qaresults.") ne %str("") %then %do;
	%let _prod_qaresults = %ms_clean_path(&_prod_qaresults) ;
	%end;

	/* Assign libnames */
	%ms_lib(dplocal, &DPLOCAL); 
	%ms_lib(msoc, &MSOC); 
	%ms_lib(infolder, &INFOLDER, options=%str(access=readonly));
	%if %str("&prod_scdm.") ne "" %then %do;
	%ms_lib(indata, &prod_scdm., options=%str(access=readonly));
	%end;
	%ms_lib(staging, &_qa_staging., options=%str(access=readonly))
	%ms_lib(qaresult, &_prod_qaresults., options=%str(access=readonly)) 

	/* Assign Macro variables*/
	%let ETL=&_ETL;
	%let DP_MinDate=&_DP_MinDate;
	%let DP_MaxDate=&_DP_MaxDate;
	%let DPID=&_DPID;   
	%let SITEID=&_SITEID;     
	%let ENRTABLE=&_ENRTABLE;    	   
	%let DEMTABLE=&_DEMTABLE;   
	%let DISTABLE=&_DISTABLE;    	
	%let DIATABLE=&_DIATABLE;     
	%let PROCTABLE=&_PROCTABLE;       
	%let ENCTABLE=&_ENCTABLE;     
	%let DEATHTABLE=&_DEATHTABLE;		      
	%let CODTABLE=&_CODTABLE;    	          
	%let LABTABLE=&_LABTABLE;         
	%let VITTABLE=&_VITTABLE;     	     
	%let IPHARMTABLE=&_IPHARMTABLE;		
	%let ITRANSTABLE=&_ITRANSTABLE;  	
	%let MILTABLE=&_MILTABLE;

	%let ccbypass = Y;
%end;

*------------------------------------------------------------------------------------------------
* If common components specified, define all file paths
*-----------------------------------------------------------------------------------------------;

%if %str("&SCC.") ne %str("") %then %do;

	/*-------------------------------------------------------------------------------------------
	* NOTES:
	*   For reasons of compatibility and standardization, paths must meet the following criteria:
	*     1- Starting with the request subdirectory, all subdirectory names must be specified in
	*        lower case (for UNIX compatibility)
	*     2- Slashes must be forward-leaning slashes '/' (for UNIX compatibility)
	*     3- Paths must end with a final forward-leaning slash '/' (extra slashes are fine)
	*------------------------------------------------------------------------------------------*/

	/* Define request specific subdirectories */
	%let DPLOCAL = &_root_dplocal.&MSReqID./dplocal/ ; 
	%let MSOC = &_root_msoc.&MSReqID./msoc/ ;            
	%let INFOLDER = &_root_inputfiles.&MSReqID./inputfiles/ ;  
	%let SASPROGRAMS = &_root_sasprograms.&MSReqID./sasprograms/ ;                 

	/* Clean up paths (e.g. convert slashes and add slashes when necessary) 
	   and validate that paths exist */
	%let DPLOCAL = %ms_clean_path(&DPLOCAL) ;
	%let MSOC = %ms_clean_path(&MSOC) ;
	%let INFOLDER = %ms_clean_path(&INFOLDER) ;
	%let SASPROGRAMS = %ms_clean_path(&SASPROGRAMS) ;

	/* Assign libnames */
	/* %ms_lib(libref-name, path-name, options=%str(options-name=options-value)) */
	%ms_lib(dplocal, &DPLOCAL) ;
	%ms_lib(msoc, &MSOC) ;
	%ms_lib(infolder, &INFOLDER, options=%str(access=readonly));

	%let ccbypass = N;

%end;

%mend;
%assignrefs();

*************************************************************************************************
*******                                  BEGIN PROGRAM                                     ******
*************************************************************************************************;

/*----------------------------------------------------------------------------------------------*/
/*      NOTE: Programmer preparing a packages includes/writes the rest of the program below     */
/*----------------------------------------------------------------------------------------------*/

*-------------------------------------------------------------------------------------------------
* 1- 
*-------------------------------------------------------------------------------------------------;

/*Macro Files Location*/          %let sasmacr=&INFOLDER./macros/;			

*Added for Routine Querying System;
**Include macro and program files ** ;  
%include "&sasmacr.combo.sas";
%include "&sasmacr.ms_agestrat.sas";
%include "&sasmacr.ms_attrition.sas";
%include "&sasmacr.ms_caresettingprincipal.sas";
%include "&sasmacr.ms_cci_elix.sas";
%include "&sasmacr.ms_cidacov.sas";
%include "&sasmacr.ms_cidadenom.sas";
%include "&sasmacr.ms_cidanum.sas";
%include "&sasmacr.ms_cidatables.sas";
%include "&sasmacr.ms_cidatablest3.sas";
%include "&sasmacr.ms_cidatablest4.sas";
%include "&sasmacr.ms_cidatablest5.sas";
%include "&sasmacr.ms_createcat.sas";
%include "&sasmacr.ms_createclaimepi.sas";
%include "&sasmacr.ms_createcontrolgroup.sas";
%include "&sasmacr.ms_createpov1.sas";
%include "&sasmacr.ms_createpov1t4.sas";
%include "&sasmacr.ms_createpov2.sas";
%include "&sasmacr.ms_createpov3.sas";
%include "&sasmacr.ms_createpov4.sas";
%include "&sasmacr.ms_createpov4t4.sas";
%include "&sasmacr.ms_createpov56.sas";
%include "&sasmacr.ms_cidambltables.sas";
%include "&sasmacr.ms_createt5episodes.sas";
%include "&sasmacr.ms_createptsmasterlist.sas";
%include "&sasmacr.ms_createriskset.sas";
%include "&sasmacr.ms_debug.sas";
%include "&sasmacr.ms_delpatients.sas";
%include "&sasmacr.ms_envelope.sas";
%include "&sasmacr.ms_episoderec.sas";
%include "&sasmacr.ms_episoderec2.sas";
%include "&sasmacr.ms_extractanniversarydate.sas";
%include "&sasmacr.ms_extractdeaths.sas";
%include "&sasmacr.ms_extractdrugs.sas";
%include "&sasmacr.ms_extractlabs.sas";
%include "&sasmacr.ms_extractmeds.sas";
%include "&sasmacr.ms_finalizeptsmasterlist.sas";
%include "&sasmacr.ms_findgap.sas";
%include "&sasmacr.ms_freezedata.sas";
%include "&sasmacr.ms_globalmacrovariables.sas";
%include "&sasmacr.ms_loopmeds.sas";
%include "&sasmacr.ms_macros.sas";
%include "&sasmacr.ms_periodsoverlap.sas";
%include "&sasmacr.ms_processinputfiles.sas";
%include "&sasmacr.ms_processwildcards.sas";
%include "&sasmacr.ms_shaveinside.sas";
%include "&sasmacr.ms_shaveoutside.sas";
%include "&sasmacr.ms_stockpiling.sas";
%include "&sasmacr.ms_surveillance.sas";
%include "&sasmacr.ms_time.sas";
%include "&sasmacr.ms_util.sas";

%include "&sasmacr.ms_covariate_adjustment.sas";
%include "&sasmacr.matchtables.sas";

%include "&sasmacr.ms_nearestneighbormatch.sas";
%include "&sasmacr.ms_appendfiles.sas";
%include "&sasmacr.ms_hdps.sas";
%include "&sasmacr.ms_runmodel.sas";

%include "&infolder.run_programs.sas";
